/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * BondLengthTableAction.cpp
 *
 *  Created on: May 9, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "bondgraph.hpp"
#include "config.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <iostream>
#include <string>

using namespace std;

#include "Actions/CommandAction/BondLengthTableAction.hpp"

// and construct the stuff
#include "BondLengthTableAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
Action::state_ptr CommandBondLengthTableAction::performCall() {
  ostringstream usage;

  // obtain information
  getParametersfromValueStorage();

  DoLog(0) && (Log() << Verbose(0) << "Using " << params.BondGraphFileName << " as bond length table." << endl);
  config *configuration = World::getInstance().getConfig();
  BondGraph *OldBG = World::getInstance().getBondGraph();

  BondGraph *BG = new BondGraph(configuration->GetIsAngstroem());
  if ((!params.BondGraphFileName.empty())
      && boost::filesystem::exists(params.BondGraphFileName)) {
    std::ifstream input(params.BondGraphFileName.string().c_str());
    if ((input.good()) && (BG->LoadBondLengthTable(input))) {
      DoLog(0) && (Log() << Verbose(0) << "Bond length table parsed successfully." << endl);
      input.close();
      CommandBondLengthTableState *state = new CommandBondLengthTableState(*OldBG, params);
      World::getInstance().setBondGraph(BG);  // Changes OldBG, hence have to copy beforehand
      return Action::state_ptr(state);
    } else {
      DoeLog(1) && (eLog()<< Verbose(1) << "Bond length table parsing failed." << endl);
      input.close();
      return Action::failure;
    }
  } else {
    DoeLog(1) && (eLog()<< Verbose(1) << "Bond length table loading failed." << endl);
    return Action::failure;
  }
}

Action::state_ptr CommandBondLengthTableAction::performUndo(Action::state_ptr _state) {
  CommandBondLengthTableState *state = assert_cast<CommandBondLengthTableState*>(_state.get());

  BondGraph *OldBG = World::getInstance().getBondGraph();
  CommandBondLengthTableState *restate = new CommandBondLengthTableState(*OldBG, params);
  World::getInstance().setBondGraph(new BondGraph(state->OldBG));  // Changes OldBG, hence have to copy beforehand

  return Action::state_ptr(restate);
}

Action::state_ptr CommandBondLengthTableAction::performRedo(Action::state_ptr _state){
  CommandBondLengthTableState *state = assert_cast<CommandBondLengthTableState*>(_state.get());

  BondGraph *OldBG = World::getInstance().getBondGraph();
  CommandBondLengthTableState *restate = new CommandBondLengthTableState(*OldBG, params);
  World::getInstance().setBondGraph(new BondGraph(state->OldBG));  // Changes OldBG, hence have to copy beforehand

  return Action::state_ptr(restate);
}

bool CommandBondLengthTableAction::canUndo() {
  return false;
}

bool CommandBondLengthTableAction::shouldUndo() {
  return false;
}
/** =========== end of function ====================== */
